@@ -31,7 +31,6 @@
 """Helper functions for //gflags."""
 
 import collections
-import itertools
 import os
 import re
 import struct
@@ -49,6 +48,7 @@ except ImportError:
 
 # pylint: disable=g-import-not-at-top
 import gflags.third_party.pep257 as pep257
+import six
 
 
 _DEFAULT_HELP_WIDTH = 80  # Default width of help output.
@@ -66,6 +66,11 @@ _MIN_HELP_WIDTH = 40  # Minimal "sane" width of help output. We assume that any
 # "least_erros >= 0.5".
 _SUGGESTION_ERROR_RATE_THRESHOLD = 0.50
 
+# Characters that cannot appear or are highly discouraged in an XML 1.0
+# document. (See http://www.w3.org/TR/REC-xml/#charsets or
+# https://en.wikipedia.org/wiki/Valid_characters_in_XML#XML_1.0)
+_ILLEGAL_XML_CHARS_REGEX = re.compile(
+    u'[\x00-\x08\x0b\x0c\x0e-\x1f\x7f-\x84\x86-\x9f\ud800-\udfff\ufffe\uffff]')
 
 # This is a set of module ids for the modules that disclaim key flags.
 # This module is explicitly added to this set so that we never consider it to
@@ -136,52 +141,58 @@ def GetCallingModule():
   return GetCallingModuleObjectAndName().module_name
 
 
-# TODO(vrusinov): we should probably just use
-# from __future__ import unicode_literals.
 def StrOrUnicode(value):
-  """Converts value to a python string or, if necessary, unicode-string."""
+  """Converts a value to a python string.
+
+  Behavior of this function is intentionally different in Python2/3.
+
+  In Python2, the given value is attempted to convert to a str (byte string).
+  If it contains non-ASCII characters, it is converted to a unicode instead.
+
+  In Python3, the given value is always converted to a str (unicode string).
+
+  This behavior reflects the (bad) practice in Python2 to try to represent
+  a string as str as long as it contains ASCII characters only.
+
+  Args:
+    value: An object to be converted to a string.
+
+  Returns:
+    A string representation of the given value. See the description above
+    for its type.
+  """
   try:
     return str(value)
   except UnicodeEncodeError:
-    return unicode(value)
+    return unicode(value)  # Python3 should never come here
 
 
-# TODO(vrusinov): this function must die.
-def _MakeXMLSafe(s):
-  """Escapes <, >, and & from s, and removes XML 1.0-illegal chars."""
-  # Note that we cannot use cgi.escape() here since it is not supported by
-  # Python 3.
-  s = s.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
-
-  # Remove characters that cannot appear in an XML 1.0 document
-  # (http://www.w3.org/TR/REC-xml/#charsets).
-  #
-  # NOTE: if there are problems with current solution, one may move to
-  # XML 1.1, which allows such chars, if they're entity-escaped (&#xHH;).
-  s = re.sub(r'[\x00-\x08\x0b\x0c\x0e-\x1f\x80-\xff]', '', s)
-  # Convert non-ascii characters to entities.  Note: requires python >=2.3
-  s = s.encode('ascii', 'xmlcharrefreplace')   # u'\xce\x88' -> 'u&#904;'
-  return s
-
-
-def WriteSimpleXMLElement(outfile, name, value, indent):
-  """Writes a simple XML element.
+def CreateXMLDOMElement(doc, name, value):
+  """Returns an XML DOM element with name and text value.
 
   Args:
-    outfile: File object we write the XML element to.
-    name: A string, the name of XML element.
+    doc: A minidom.Document, the DOM document it should create nodes from.
+    name: A string, the tag of XML element.
     value: A Python object, whose string representation will be used
-      as the value of the XML element.
-    indent: A string, prepended to each line of generated output.
+      as the value of the XML element. Illegal or highly discouraged xml 1.0
+      characters are stripped.
+
+  Returns:
+    An instance of minidom.Element.
   """
-  value_str = StrOrUnicode(value)
+  s = StrOrUnicode(value)
+  if six.PY2 and not isinstance(s, unicode):
+    # Get a valid unicode string.
+    s = s.decode('utf-8', 'ignore')
   if isinstance(value, bool):
     # Display boolean values as the C++ flag library does: no caps.
-    value_str = value_str.lower()
-  # TODO(vrusinov): we should use some lightweight built-in xml library instead
-  # of hand-crafting xml.
-  safe_value_str = _MakeXMLSafe(value_str)
-  outfile.write('%s<%s>%s</%s>\n' % (indent, name, safe_value_str, name))
+    s = s.lower()
+  # Remove illegal xml characters.
+  s = _ILLEGAL_XML_CHARS_REGEX.sub(u'', s)
+
+  e = doc.createElement(name)
+  e.appendChild(doc.createTextNode(s))
+  return e
 
 
 def GetHelpWidth():
@@ -202,27 +213,6 @@ def GetHelpWidth():
     return _DEFAULT_HELP_WIDTH
 
 
-def GetMainModule():
-  """Get the program name.  Do no use, this function is likely to be deleted.
-
-  Returns:
-    str, The name of the program.
-  """
-  # This is a poorly named internal API.  In the past it used to go crazy and
-  # walk up the sys._getframe stack until it hit the top and call
-  # GetModuleObjectAndName on that...  Meanwhile GetModuleObjectAndName was
-  # always returning sys.argv[0] instead of anything related to the module as
-  # the name of the top module is always __main__.  That logic was tripping
-  # up an embedded Python launcher that uses a tiny module above __main__
-  # to read, compile and exec __main__.  Solution: Simplify to sys.argv[0]!
-  #
-  # Flags only uses this to determine what flags to show in the short help
-  # text and what to show as the program name.
-  #
-  # TODO(vrusinov): Delete this function entirely now that it is trivial.
-  return sys.argv[0]
-
-
 def GetFlagSuggestions(attempt, longopt_list):
   """Get helpful similar matches for an invalid flag."""
   # Don't suggest on very short strings, or if no longopts are specified.
@@ -279,11 +269,11 @@ def _DamerauLevenshtein(a, b):
   return Distance(a, b)
 
 
-def TextWrap(text, length=None, indent='', firstline_indent=None, tabs='    '):
+def TextWrap(text, length=None, indent='', firstline_indent=None):
   """Wraps a given text to a maximum line length and returns it.
 
-  We turn lines that only contain whitespace into empty lines. We keep new
-  lines.
+  It turns lines that only contain whitespace into empty lines, keeps new lines,
+  and expands tabs using 4 spaces.
 
   Args:
     text:             str, Text to wrap.
@@ -291,7 +281,6 @@ def TextWrap(text, length=None, indent='', firstline_indent=None, tabs='    '):
                       If this is None then use GetHelpWidth()
     indent:           str, Indent for all but first line.
     firstline_indent: str, Indent for first line; if None, fall back to indent.
-    tabs:             ste, Replacement for tabs.
 
   Returns:
     Wrapped text.
@@ -312,11 +301,7 @@ def TextWrap(text, length=None, indent='', firstline_indent=None, tabs='    '):
   if len(firstline_indent) >= length:
     raise ValueError('Length of first line indent exceeds length')
 
-  # TODO(vrusinov): tabs param is not used in flags, remove it altogether and
-  # let textwrap use expandtabs().
-  if tabs is None:
-    tabs = ' '
-  text = text.replace('\t', tabs)
+  text = text.expandtabs(4)
 
   result = []
   # Create one wrapper for the first paragraph and one for subsequent
@@ -359,7 +344,7 @@ def FlagDictToArgs(flag_map):
   Yields:
     sequence of string suitable for a subprocess execution.
   """
-  for key, value in flag_map.iteritems():
+  for key, value in six.iteritems(flag_map):
     if value is None:
       yield '--%s' % key
     elif isinstance(value, bool):
@@ -373,7 +358,7 @@ def FlagDictToArgs(flag_map):
     else:
       # Now we attempt to deal with collections.
       try:
-        yield '--%s=%s' % (key, ','.join(itertools.imap(str, value)))
+        yield '--%s=%s' % (key, ','.join(str(item) for item in value))
       except TypeError:
         # Default case.
         yield '--%s=%s' % (key, value)
@@ -414,3 +399,32 @@ def IsRunningTest():
       'pytest',
   }
   return bool(test_modules & modules)
+
+
+# TODO(b/31830082): Migrate all users to PEP8-style methods and remove this.
+def define_both_methods(class_name, class_dict, old_name, new_name):  # pylint: disable=invalid-name
+  """Function to help CamelCase to PEP8 style class methods migration.
+
+  For any class definition:
+      1. Assert it does not define both old and new methods,
+         otherwise it does not work.
+      2. If it defines the old method, create the same new method.
+      3. If it defines the new method, create the same old method.
+
+  Args:
+    class_name: the class name.
+    class_dict: the class dictionary.
+    old_name: old method's name.
+    new_name: new method's name.
+
+  Raises:
+    AssertionError: raised when the class defines both the old_name and
+        new_name.
+  """
+  assert old_name not in class_dict or new_name not in class_dict, (
+      'Class "{}" cannot define both "{}" and "{}" methods.'.format(
+          class_name, old_name, new_name))
+  if old_name in class_dict:
+    class_dict[new_name] = class_dict[old_name]
+  elif new_name in class_dict:
+    class_dict[old_name] = class_dict[new_name]
